import { Http, Loader, Metadata, Singleton } from '@zeedhi/core';
import { OperadorController } from '@/controllers/Operador/OperadorController';
import { OperadorService } from '@/services/OperadorService';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Form, IGridEvent, LoadingService, Modal, ModalService } from '@zeedhi/common';
import { PmLibAlertService, PmLibDialogService, PmLibHttp } from '@zeedhi/pm-lib';
import AclMessages from '@/Util/AclMessages';
import { Date as DateInput } from '@zeedhi/common/types/components/zd-date/date';

@Singleton
export class TabPerfilAcessoController {
	public operadorControler: OperadorController;

	public operadorService: OperadorService;

	constructor() {
		this.operadorControler = Loader.getInstance('OperadorController');
		this.operadorService = new OperadorService();
	}

	private getInstanceOperadorPerfilDeAcessoGrid() {
		return Metadata.getInstance<TekGrid>('operadorPerfilDeAcessoGrid');
	}

	private getInstanceAddPerfilByOperadorGrid() {
		return Metadata.getInstance<TekGrid>('addPerfilByOperadorGrid');
	}

	/** Grid Component */

	public getCurrentNrparcnegocio(): string {
		return this.operadorControler.currentOperador.NRPARCNEGOCIO;
	}

	public async addClickOperadorPerfilDeAcessoGrid() {
		await this.createAddPerfilByOperadorModal();
		await this.addPerfilAcessoByOperadorModal.show();
		const addGrid = this.getInstanceAddPerfilByOperadorGrid();
		const nrparcnegocio = this.getCurrentNrparcnegocio();
		await addGrid.datasource.addFilter('NRPARCNEGOCIO', nrparcnegocio);
	}

	public async beforeSaveOperadorPerfilDeAcessoGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const grid = this.getInstanceOperadorPerfilDeAcessoGrid();
		await PmLibHttp.$put('/perfacessparc', grid.getEditedRows());
		PmLibAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
		await grid.reload();
		await grid.cancelEditedRows();
	}

	public async beforeDeleteOperadorPerfilDeAcessoGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const confirm = await PmLibDialogService.confirmDialog(AclMessages.DELETE_DEFAULT);
		if (confirm) {
			this.deletePerfilacesso();
		}
	}

	public async deletePerfilacesso() {
		const grid = this.getInstanceOperadorPerfilDeAcessoGrid();
		await PmLibHttp.$delete('/perfacessparc', { data: grid.selectedRows });
		PmLibAlertService.showSuccess(AclMessages.SUCCESS_DELETE_DEAFULT);
		await grid.reload();
	}

	/** Form Add Modal Component */

	public addPerfilAcessoByOperadorModal!: Modal;

	public addPerfilAcessoByOperadorModalPath: string = 'components/operador/detail/tabs/operadorPerfilDeAcesso/modais/addPerfilByOperador';

	public async createAddPerfilByOperadorModal() {
		this.addPerfilAcessoByOperadorModal = await ModalService.createFromJson(this.addPerfilAcessoByOperadorModalPath, true);
	}

	public async closeAddPerfilByOperadorModal() {
		this.addPerfilAcessoByOperadorModal.hide();
		this.addPerfilAcessoByOperadorModal?.destroy();
		await this.getInstanceOperadorPerfilDeAcessoGrid().reload();
	}

	public get validateSaveButtonAddPerfilByOperadorModal(): boolean {
		const addGrid = this.getInstanceAddPerfilByOperadorGrid();
		return !addGrid.selectedRows.length;
	}

	public async saveAddOperadorByPerfilModal() {
		const addGrid = this.getInstanceAddPerfilByOperadorGrid();
		const addPerfilModalForm = Metadata.getInstance<Form>('addPerfilByOperadorForm');
		const dtValidateField = addPerfilModalForm.getChildInstance<DateInput>('DTVALIDATE');
		const { selectedRows } = addGrid;
		selectedRows.forEach((row) => {
			row.DTVALIDADE = dtValidateField.value;
			row.NRPARCNEGOCIO = this.getCurrentNrparcnegocio();
			row.CDOPERADOR = this.operadorControler.currentOperador.CDOPERADOR;
			row.NMOPERADOR = this.operadorControler.currentOperador.NMOPERADOR;
		});

		try {
			LoadingService.show();
			await Http.post('/perfacessparc', selectedRows);
			LoadingService.hide();
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_ADD_OPERADORES);
			dtValidateField.setDateValue('');
			addGrid.selectedRows = [];
			await addGrid.reload();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}
}
